searchData={"items":[{"type":"behaviour","title":"EctoHooks","doc":"Module exposing various utility functions for interacting with, and introspecting\nthe state of EctoHooks.\n\nFor more extensive usage instructions, please see associated documentation for\n`EctoHooks.Repo`.","ref":"EctoHooks.html"},{"type":"macro","title":"EctoHooks.__using__/1","doc":"Alternative interface for initializing EctoHooks when `use`-ed in a module as follows:\n\n```elixir\ndef MyApp.Repo do\n  use Ecto.Repo,\n    otp_app: :my_app,\n    adapter: Ecto.Adapters.Postgres\n\n  use EctoHooks\nend\n```\n\nPlease see `EctoHooks.Repo` for more information about EctoHooks usage, examples,\nand considerations.","ref":"EctoHooks.html#__using__/1"},{"type":"callback","title":"EctoHooks.after_delete/2","doc":"","ref":"EctoHooks.html#c:after_delete/2"},{"type":"callback","title":"EctoHooks.after_get/2","doc":"","ref":"EctoHooks.html#c:after_get/2"},{"type":"callback","title":"EctoHooks.after_insert/2","doc":"","ref":"EctoHooks.html#c:after_insert/2"},{"type":"callback","title":"EctoHooks.after_update/2","doc":"","ref":"EctoHooks.html#c:after_update/2"},{"type":"callback","title":"EctoHooks.before_delete/1","doc":"","ref":"EctoHooks.html#c:before_delete/1"},{"type":"callback","title":"EctoHooks.before_insert/1","doc":"","ref":"EctoHooks.html#c:before_insert/1"},{"type":"callback","title":"EctoHooks.before_update/1","doc":"","ref":"EctoHooks.html#c:before_update/1"},{"type":"function","title":"EctoHooks.disable_hooks/1","doc":"Disables EctoHooks running for all future Repo operations in the current process.\n\nInternally, this is called with the options `[global: false]` which gets\nautomatically cleared after triggering any `Ecto.Repo` callback.","ref":"EctoHooks.html#disable_hooks/1"},{"type":"function","title":"EctoHooks.enable_hooks/1","doc":"Ensures EctoHooks run for all future Repo operations in the current process.\n\nInternally, we use it to scope re-enabling hooks per execution, as we disabled\nthem as a mitigation for infinitely looping hooks caused by hooks triggering\nother hooks. This happens when provided `[global: false]`, which should only be\ndone internally.","ref":"EctoHooks.html#enable_hooks/1"},{"type":"function","title":"EctoHooks.hooks_enabled?/0","doc":"Returns a boolean indicating if any `before_*` or `after_*` hooks are allowed\nto execute in the current process.\n\nNote that calls to `Ecto.Repo` callbacks may reset or change the state of this flag,\nand that this function is primarily exposed for introspection or debugging purposes.","ref":"EctoHooks.html#hooks_enabled?/0"},{"type":"function","title":"EctoHooks.hooks_ref_count/0","doc":"Utility function which returns the \"nesting\" of the current EctoHooks context.\n\nBy default, every hook will \"acquire\" an EctoHook context and increment a ref count.\nThese ref counts are automatically decremented once a hook finishes running.\n\nThis is provided as a lower level alternative the `enable_hooks/1`, `disable_hooks/1`,\nand `hooks_enabled?/0` functions.","ref":"EctoHooks.html#hooks_ref_count/0"},{"type":"function","title":"EctoHooks.in_hook?/0","doc":"Utility function which returns true if currently executing inside the context of an\nEcto Hook.","ref":"EctoHooks.html#in_hook?/0"},{"type":"module","title":"EctoHooks.Delta","doc":"Defines the struct which is given as the 2nd argument to any `after_*` hooks.\n\nContains metadata which may be useful for introspecting or otherwise conditionally\nrunning hooks only when certain conditions are met.\n\nThe following information is present in an `EctoHooks.Delta`:\n\n- The `Ecto.Repo` callback that triggered the hook\n- The name of the current hook (useful if delegating to private functions in your schema)\n- The `Ecto.Queryable` passed into the triggering `Ecto.Repo` callback if any\n- The `Ecto.Changeset` passed into the triggering `Ecto.Repo` callback if any","ref":"EctoHooks.Delta.html"},{"type":"function","title":"EctoHooks.Delta.new!/3","doc":"","ref":"EctoHooks.Delta.html#new!/3"},{"type":"type","title":"EctoHooks.Delta.hook/0","doc":"","ref":"EctoHooks.Delta.html#t:hook/0"},{"type":"type","title":"EctoHooks.Delta.repo_callback/0","doc":"","ref":"EctoHooks.Delta.html#t:repo_callback/0"},{"type":"type","title":"EctoHooks.Delta.t/0","doc":"","ref":"EctoHooks.Delta.html#t:t/0"},{"type":"module","title":"EctoHooks.Middleware.After","doc":"Module containing implementation for all `after_` hooks","ref":"EctoHooks.Middleware.After.html"},{"type":"module","title":"EctoHooks.Middleware.Before","doc":"Module containing implementation for all `before_` hooks","ref":"EctoHooks.Middleware.Before.html"},{"type":"module","title":"EctoHooks.Repo","doc":"This module is intended to replace any usage of `use Ecto.Repo`. When this is done,\nany of the following `Ecto.Repo` callbacks will execute any defined `before_*`\nor `after_*` hooks in your Ecto schemas before returning.","ref":"EctoHooks.Repo.html"},{"type":"module","title":"Setup and initialization - EctoHooks.Repo","doc":"Simply `use` this module wherever you would `use Ecto.Schema`. Any provided\noptions are currently forwarded to `use Ecto.Repo`. In the future, EctoHooks\nmay provide additional functionality based on given options.\n\nExample Usage:\n\n```elixir\ndefmodule MyApp.Repo do\n  use EctoHooks.Repo,\n    otp_app: :my_app,\n    adapter: Ecto.Adapters.Postgres\nend\n\ndefmodule MyApp.User do\n  use Ecto.Schema\n  require Logger\n\n  schema \"users\" do\n    field :first_name, :string\n    field :last_name, :string\n\n    field :full_name, :string, virtual: true\n  end\n\n  def before_insert(changeset) do\n    Logger.warning(\"inserted a new user...\")\n    changeset\n  end\n\n  def after_get(%__MODULE__{} = user, %EctoHooks.Delta{}) do\n    %__MODULE__{user | full_name: user.first_name <> \" \" <> user.last_name}\n  end\nend\n```\n\nAs an aside, for documentation and potential code-completion purposes, you may also\nannotate your `Ecto.Schema` modules with the following:\n\n```elixir\ndefmodule MyApp.User do\n  use Ecto.Schema\n  @behaviour EctoHooks\n\n  ...\n\n\n  @impl EctoHooks\n  def after_get(%__MODULE__{} = user, %EctoHooks.Delta{}) do\n    %__MODULE__{user | full_name: user.first_name <> \" \" <> user.last_name}\n  end\nend\n```","ref":"EctoHooks.Repo.html#module-setup-and-initialization"},{"type":"module","title":"Usage - EctoHooks.Repo","doc":"#","ref":"EctoHooks.Repo.html#module-usage"},{"type":"module","title":"After Hooks - EctoHooks.Repo","doc":"Once initialized, the following `Ecto.Repo` callbacks will trigger the listed\n`after_*` hooks.\n\nExecutes `after_get/2` if defined in schema:\n- `all/2`\n- `get/3`\n- `get!/3`\n- `get_by/3`\n- `get_by!/3`\n- `one/2`\n- `one!/2`\n- `reload/2`\n- `reload!/2`\n- `preload/3`\n\nExecutes `after_delete/2` if defined in schema:\n- `delete/2`\n- `delete!/2`\n\nExecutes `after_insert/2` if defined in schema:\n- `insert/2`\n- `insert!/2`\n\nExecutes `after_update/2` if defined in schema:\n- `update/2`\n- `update!/2`\n\nExecutes `after_insert/2` or to `after_update/2` if defined in schema:\n- `insert_or_update/2`\n- `insert_or_update!/2`\n\nPlease note that the result of an `after_*` hook is the result which is intended\nto be returned by any of the above `Ecto.Repo` callbacks. All `after_*` hooks are\nexpected to abide by the following typespec:\n\n```elixir\n@spec after_*(schema_struct :: MyApp.User.t(), delta :: EctoHooks.Delta.t()) :: MyApp.User.t()\n```\n\nAnd any `Ecto.Repo` callback which expects to return `{:ok, result}` over `result`\nwill do so automatically.\n\nTake care to not break expected `Ecto.Repo` callback semantics as bugs resulting\nfrom these can prove difficult to diagnose and track down. As a rule of thumb, you\nshould always aim to return the same type of argument passed into your defined hooks.\n\n#","ref":"EctoHooks.Repo.html#module-after-hooks"},{"type":"module","title":"Before Hooks - EctoHooks.Repo","doc":"Once initialized, the following `Ecto.Repo` callbacks will trigger the listed\n`before_*` hooks.\n\nExecutes `before_delete/1` if defined in schema:\n- `delete/2`\n- `delete!/2`\n\nExecutes `before_insert/1` if defined in schema:\n- `insert/2`\n- `insert!/2`\n\nExecutes `before_update/1` if defined in schema:\n- `update/2`\n- `update!/2`\n\nExecutes `before_insert/1` or to `before_update/1` if defined in schema:\n- `insert_or_update/2`\n- `insert_or_update!/2`\n\nPlease note that the result of an `before_*` hook is usually the `Ecto.Queryable`\nwhich is passed as the first parameter to any of the above `Ecto.Repo` callbacks.\nAll `before_*` hooks are expected to abide by the following typespec:\n\n```elixir\n@spec before_*(queryable :: Ecto.Queryable.t()) :: Ecto.Queryable.t()\n```\n\n#","ref":"EctoHooks.Repo.html#module-before-hooks"},{"type":"module","title":"Considerations - EctoHooks.Repo","doc":"EctoHooks was initially designed to help with centralizing and de-duplicating\nlogic to resolve virtual fields when working with Ecto schemas. However, since\nEctoHooks allows you to call arbitrary functions in response to `Ecto.Repo` callbacks,\nit can be used for much more.\n\nAll defined hooks are also *executed synchronously*, immediately before or after\ncalling your `Ecto.Repo` callback. To try and mitigate the potential for infinite\nloops caused by hooks triggering other hooks, by default, EctoHooks will only\ntrigger a hook if a parent function in *the current process stacktrace* is not another\nhook.\n\nAs a result, one should take *great care* when spawning processes which may result\nin calling `Ecto.Repo` callbacks, as there is currently no mitigation for infinite\nloops provided.\n\nIf you wish to opt in, opt out, or introspect the state of these mitigations, please\nsee the documentation for the following functions: `EctoHooks.enable_hooks/1`,\n`EctoHooks.disable_hooks/1`, `EctoHooks.hooks_enabled?/0` and `EctoHooks.in_hook?/0`.\n\nThis is a very powerful tool, and admittedly quite hidden and hard to track down.\nTrying to avoid adding too much business logic in your EctoHooks is wise for both\nperformance reasons (as hooks are executed on every request), and maintainability\nreasons.","ref":"EctoHooks.Repo.html#module-considerations"},{"type":"macro","title":"EctoHooks.Repo.__using__/1","doc":"Enables EctoHooks integration in your `Ecto.Repo` module when `use`-ed.","ref":"EctoHooks.Repo.html#__using__/1"}],"content_type":"text/markdown"}